#include "action.h"

// nonmember functions -------------------------------------------------------

extern long double choose_random_value(void);

// member functions ---------------------------------------------------------

Action::~Action(void)
{
}

Action::Action(void)
{
  value = 0; 
  fatigue = 0; fatigueGain = 0; fatigueLoss = 0;

  inhibitorCount = 0;  endovarCount = 0; blockCount = 0;

  startIndex = 0;
  red = 255; green = 255; blue = 255;
}

void
Action::updateFatigue(void)
{
  // value dependent rate of increase in fatigue for the activity
  if (value > 0) fatigue += fatigueGain*value;
  else fatigue -= fatigue*fatigueLoss;
  if (fatigue < 0) fatigue = 0; else if (fatigue > 1) fatigue = 1;

}

void
Action::updateValue(void)
{
  double comb = 0; 

  // if a blocking conditions is satisfied set value to 0 and exit
  for (int i=0;i<blockCount;i++) {
    Relmech &block = *blocks[i];
    if (block.isActive() == flags[i]) {
      value = 0; 
      valueHistory[startIndex] = value;
      fatigueHistory[startIndex] = fatigue;
      startIndex = (startIndex + 1) % 100;
      return;
    }
  }

  // compute the sum of endogenous varibables
  for (i=0;i<endovarCount;i++) {
    Endovar &endovar = *endovars[i];
    comb += endovarGains[i]*endovar.getValue();
  }

  // compute the sum of releasing mechanisms (only if comp <= 0)
  //  if (comb >= 0) comb += sumReleases();

  // compute the action value
  value = (1.0 - fatigue) * comb;
  
  // compute the inhibition from other behaviors
  for (i=0;i<inhibitorCount;i++) {
    Action &inhibitor = *inhibitors[i];
    value -= inhibitorGains[i]*inhibitor.value;
  }
  
  if (value < 0) value = 0;
  //  else cerr << actionName << " " << value << endl;

  // finally, check the blocks
  for (i=0;i<blockCount;i++) {
    if (blocks[i]->isActive() == flags[i]) value = 0;
  }

  valueHistory[startIndex] = value;
  fatigueHistory[startIndex] = fatigue;
  startIndex = (startIndex + 1) % 100;
}

void
Action::addInhibitor(Action *inh, double gainFactor)
{
  assert(inh);
  inhibitors[inhibitorCount] = inh;
  inhibitorGains[inhibitorCount] = gainFactor;
  inhibitorCount++;
}

void
Action::addEndovar(Endovar *endovar, double gainFactor)
{
  assert(endovar);
  endovars[endovarCount] = endovar;
  endovarGains[endovarCount] = gainFactor;
  endovarCount++;
}

void
Action::addBlock(Relmech *block, int flag)
{
  assert(block);
  blocks[blockCount] = block;
  flags[blockCount] = flag;
  blockCount++;
}

void
Action::setColor(GLubyte r, GLubyte g, GLubyte b)
{
  red = r; green = g; blue = b;
}

void
Action::drawValue(void)
{
  int i;  double x = 0; 

  // draw the action value
  glColor3ub(red,green,blue);
  glBegin(GL_LINE_STRIP);
  for (i=startIndex;i<100;i++) { 
    glVertex3d(x,valueHistory[i],0); x += 1.0; 
  }
  for (i=0;i<startIndex;i++) { 
    glVertex3d(x,valueHistory[i],0); x += 1.0;
  }
  glEnd();

}

void
Action::drawFatigue(void)
{
  int i;  double x = 0; 

  // draw the fear level
  glColor3ub(red,green,blue);
  glBegin(GL_LINE_STRIP);
  for (i=startIndex;i<100;i++) { 
    glVertex3d(x,fatigueHistory[i],0); x += 1.0; 
  }
  for (i=0;i<startIndex;i++) { 
    glVertex3d(x,fatigueHistory[i],0); x += 1.0;
  }
  glEnd();
  
}
