#include "endovar.h"

// nonmember functions -------------------------------------------------------

// member functions ---------------------------------------------------------

Endovar::~Endovar(void)
{
}

Endovar::Endovar(void)
{
  value = 0;   minValue = 0; autoGain = 0; 

  startIndex = 0;
  actionCount = 0;

  red = 255; green = 255; blue = 255;
  
}

void
Endovar::setValue(double val)
{
  assert (val <= 1.0 && val >= minValue);
  value = val;
}

void
Endovar::setMinValue(double val)
{
  assert (val <= 1.0 && val >= 0);
  minValue = val;
}

void
Endovar::updateValue(void)
{
  value += autoGain;

  // compute endogenous gain due to associated activities
  for (int i=0;i<actionCount;i++) {
    double endoGain = actions[i]->getValue()*gains[i];
    value += endoGain;
  }

  // compute endogenous change due to associated releasing mechanisms
  for (i=0;i<relmechCount;i++) {
    if (relmechs[i]->isActive()) value += activeGains[i];
    else value += inactiveGains[i];
  }

  if (value < minValue) value = minValue; 
  else if (value > 1.0) value = 1.0;

  valueHistory[startIndex] = value;
  startIndex = (startIndex + 1) % 100;
}

void
Endovar::addAction(Action *action, double gain)
{
  assert (actionCount < BIRD_MAX_ACTIONS);
  assert(action);
  actions[actionCount] = action;
  gains[actionCount] = gain;
  actionCount++;
}

void
Endovar::addRelmech(Relmech *relmech, double activeGain, double inactiveGain)
{
  assert (relmechCount < BIRD_MAX_RELMECHS);
  assert(relmech);
  relmechs[relmechCount] = relmech;
  activeGains[relmechCount] = activeGain;
  inactiveGains[relmechCount] = inactiveGain;
  relmechCount++;
}

void
Endovar::setColor(GLubyte r, GLubyte g, GLubyte b)
{
  red = r; green = g; blue = b;
}

void
Endovar::drawGL(void)
{
  double x = 0; 
  glColor3ub(red,green,blue);
  glBegin(GL_LINE_STRIP);
  for (int i=startIndex;i<100;i++) { 
    glVertex3d(x,valueHistory[i],0); x += 1.0; 
  }
  for (i=0;i<startIndex;i++) { 
    glVertex3d(x,valueHistory[i],0); x += 1.0;
  }
  glEnd();
}
